const {writeData2File} = require('@pactflow/pact-msw-adapter/dist/utils/utils')
const {toQueryString} = require('@s-ui/js/lib/string')

const writerFactory = providers => (path, data) => {
  const {interactions, provider} = data
  const {name} = provider

  data.interactions = interactions.map(interaction => {
    const {request} = interaction
    const {path, query} = request
    const definedInteraction = providers[name].find(
      ({endpoint, query: definedQuery}) => {
        const matchedEndpoint = endpoint === path

        if (query) {
          return matchedEndpoint && query === toQueryString(definedQuery)
        }
        return matchedEndpoint
      }
    )
    const {description, state: providerState} = definedInteraction

    return {
      ...interaction,
      description,
      providerState
    }
  })

  writeData2File(path, data)
}

const mapProviders = providers =>
  Object.keys(providers).reduce((reducedProviders, key) => {
    const endpoints = providers[key].map(({endpoint, query}) =>
      query ? `${endpoint}?${toQueryString(query)}` : endpoint
    )

    return {
      ...reducedProviders,
      [key]: endpoints
    }
  }, {})

const getContractTests = ({apiUrl, providers}) =>
  Object.keys(providers).reduce(
    (tests, key) => [...tests, ...providers[key]],
    []
  )

module.exports = {writerFactory, mapProviders, getContractTests}
